```{eval-rst}
.. meta::
   :description: В курсе рассматривается ряд задач теплофизики и необходимые для их решения вычислительные методы. Все задачи мы сопровождаем практикой, то есть, программированием.
   :keywords: теплофизика, термодинамика, фазовое равновесие, вычислительная математика, вычматы, программирование, алгоритмы, julia
```


# О практикуме

```{epigraph}
Sapere aude (Дерзай знать)

-- Девиз Московского Физико-Технического Института
```

**Практикум по вычислительной теплофизики** читается на кафедре физики высокотемпературных процессов [Московского Физико-Технического Института](https://mipt.ru/).
Авторы курса [Степан Захаров](https://scholar.google.ru/citations?user=xvp7Z9oAAAAJ) и [Василий Писарев](https://scholar.google.ru/citations?user=ZLTxwC8AAAAJ).

В области ответственности теплофизики находится целый ряд задач.

**Уравнения состояния.** Вывод новых уравнений состояния и параметризация существующих.

**Фазовое равновесие.** Нахождение термодинамически стабильного состояния системы при данных параметрах.

**Химические реакции.** Расчёт скоростей реакций и их динамики.

**Гидродинамика.** (Флюидодинамика) Уравнения гидродинамики (например, перенос тепла) требуют для решения достижения термодинамики. В некоторых задачах термодинамические эффекты (а не механические) играют решающую роль (например, фазовые переходы).

**Коэффициенты переноса.** Расчёт коэффициентов переноса для конкретных веществ или смесей. Примеры коэффициентов: теплопроводность, вязкость, диффузия, скорость звука, поверхностное натяжение. Без этих коэффициентов, как правило, не обходятся задачи гидродинамики. Без них также сложно оценить, какие течения преобладают, например, диффузивное или конвективное, ламинарное или турбулентное.

В свою очередь, каждая из этих задач имеет аналитическую постановку.
Вот несколько задач, к которым так или иначе сводятся вышеперечисленные.

- Решение нелинейного скалярного уравнения $f(x) = 0$;
- Интегрирование функции $\int f \diff x$;
- Решение системы линейных $\mathbf{A}\mathbf{x} = \mathbf{b}$ уравнений;
- Решение нелинейного векторного уравнения $f(\mathbf{x}) = 0$;
- Решение системы нелинейных $\mathbf{A}(\mathbf{x})\mathbf{x} = \mathbf{b}(\mathbf{x})$ уравнений $(\mathbf{F}(\mathbf{x}) = \mathbf{0})$;
- Решение обыкновенных дифференциальных уравнений и систем $\diff f / \diff x = g(x)$;
- Решение уравнений и систем уравнений в частных производных $f_t + f_x = 0$, $f_t + (\kappa f_x)_x = 0$, $f_{xx} = g(x)$;
- Задача условной и безусловной оптимизации $\mathbf{x}^*\in\Omega: f(\mathbf{x}^*) = \min_\Omega f(\mathbf{x})$.

*Однако, есть трудность*: реальные системы описываются нелинейными уравнениями и зависимостями, поэтому только в редких частных случаях можно аналитически решить задачи выше.
В общем случае, эти задачи решаются вычислительными (дискретными) методами.

В курсе мы рассмотрим некоторые задачи теплофизики и необходимые для их решения вычислительные методы.
Все задачи мы сопровождаем практикой, то есть, программированием.


```{tip}
Сообщить об ошибке в материалах курса можно через [Телеграмм](https://red_deer.t.me) или GitHub {guilabel}`Иконка сверху → Open issue`.

Если вы хотите воспользоваться материалами курса не в учебных целях, то обратитесь к авторам курса.
```
