```{eval-rst}
.. meta::
   :description: В этом разделе мы рассмотрим задачу численного интегрирования функции и основные методы её решения.
   :keywords: численное интегрирование, квадратура, квадратурная формула, вычислительная математика, вычматы
```

# Численное интегрирование

В математическом анализе рассматривается задача нахождения определённого интеграла

```{math}
:label: int

I = \int_a^b f(x)\diff x,
```

где $f(x)$ будем считать непрерывной и без особенностей на $[a, b]$.

Однако, не для любой подынтегральной функции существует первообразная, выражаемая через элементарные функции.

```{index} формула; квадратурная 
```
```{index} квадратура; формула, квадратура; вес, квадратура; узел
```
:::{proof:definition} Квадратурная формула
Задача численного интегрирования сводится к приближённому вычислению интеграла {eq}`int` и выражается в виде **квадратурной формулы**

```{math}
:label: quadrature

\int_a^b f(x)\diff x \approx \sum_{i=0}^n w_i f(x_i),
```

здесь отрезок $[a, b]$ был разбит $n+1$ точкой на $n$ интервалов. Точки $x_i \in [a, b]$ называют **узлами** (*nodes*), а $w_i$ &ndash; **весами** (*weights*) или *коэффициентами* квадратурной формулы.
:::

```{index} погрешность; квадратурной формулы, квадратура; погрешность
```
:::{proof:definition} Погрешность квадратурной формулы
Разность

```{math}
\tau_f(n) = \int_a^b f(x)\diff x - \sum_{i=0}^n w_i f(x_i)
```

называют **погрешностью квадратурной формулы** (*truncation error*). Погрешность зависит от числа узлов, их расположения и выбора весовых коэффициентов $w_i$.
:::

Введём сетку $\omega_h = \{x_0 = a < x_1 < \dots < x_N = b\}$, тогда интеграл {eq}`int` можно разбить на сумму интегралов по отрезкам $[x_{i-1}, x_i]$

```{math}
:label: quadrature_composite

\int_a^b f(x)\diff x = \sum_{i=1}^n \int_{x_{i-1}}^{x_i} f(x)\diff x.
```

```{index} формула; квадратурная составная, квадратура; формула составная
```
:::{proof:definition} Составная квадратурная формула
Если на каждом из отрезков $[x_{i-1}, x_i]$ интеграл приближается квадратурной формулой в форме {eq}`quadrature`, то выражение {eq}`quadrature_composite` называют **составной квадратурной формулой**.
:::

Популярны составные квадратурные формулы для частного случая равномерной сетки $\omega_h = \{x_i = a + ih, i = 1, \dots, N, h = \frac{b - a}{N}\}$.

Видно, что для вывода квадратурных формул достаточно рассмотреть в качестве базового случая интегрирование по отрезку {eq}`quadrature`, а затем при необходимости воспользоваться формулой {eq}`quadrature_composite`.

Также для вывода квадратурных формул зачастую удобно рассматривать интегрирование на отрезках $[0, 1]$ или $[-d/2, d/2]$, а для интегрирования на произвольном отрезке $[a, b]$ применять преобразование

```{math}
\int\limits_{a}^{b} f(x)\diff x = (b - a)\int\limits_{0}^{1} f(a + x(b - a)) \diff x = \frac{b - a}{d}\int\limits_{-d/2}^{d/2} f\Big(a + (x + 1)\frac{b - a}{d}\Big) \diff x,
```

В этом разделе рассмотрим некоторые из квадратурных формул.
