---
myst:
  substitutions:
    urldownload: "https://julialang.org/downloads/"
    wintermdwnld: "<img src=\"https://www.microsoft.com/favicon.ico%3Fv2\" style=\"height: 1em;\" alt=\"⊞\"> [Windows Terminal](https://aka.ms/terminal)"
    julia_logo: "<img src=\"https://raw.githubusercontent.com/JuliaLang/julia-logo-graphics/master/images/julia-dots.svg\" style=\"height: 1em;\">"
---

```{eval-rst}
.. meta::
   :description: Установка языка компилятора языка Julia на Windows, Linux-системы и MacOS.
   :keywords: julia, установка, инструкция
```

# Установка

Ниже расположены краткие инструкции по установке исполнителя языка Julia для Linux-based операционных систем (ОС), MacOS и Windows. При возникновении неясностей, обратитесь к [официальной инструкции](https://julialang.org/downloads/platform/) для вашей ОС.

```{warning}
Инструкция ниже написана для версии языка `1.6.2`.
Скорее всего, вам потребуется изменить некоторые команды, заменив в них `1.6.2` на скачанную вами версию языка.
```

## Linux-based ОС

**1. Скачать последнюю версию интерпретатора**

На сайте загрузок {{ urldownload }} найти в таблице свою систему и архитектуру
(скорее всего, это {guilabel}`Generic Linux on x86 → 64-bit`).

:::{warning}
Использовать пакеты `julia` из репозитория дистрибутива настоятельно не рекомендуется, поскольку в репозиториях сборка часто неверно сконфигурирована, из-за чего сторонние библиотеки для Julia могут отказаться работать.
:::

**2. Распаковать скачанный архив**

(предполагаем, что скачанный архив лежит непосредственно в домашней директории)

```console
$ tar -xzf julia-1.6.2-linux-x86_64.tar.gz
```

(здесь и далее команды в терминале помечаются `$`)

После этого в домашней директории должна появиться директория {file}`julia-1.6.2`, где находится исполняемый файл интерпретатора и необходимые для его работы библиотеки.

**3. Добавление ссылки для удобного вызова**

Рекомендуется вместо добавления в `PATH` директории, в которой лежит исполняемый файл, добавить символьную ссылку на этот файл в директорию, которая уже в `PATH`. В большинстве дистрибутивов директория {file}`~/bin`, если она существует, автоматически добавляется в `PATH`, что позволяет добавить ссылку, даже не имея прав администратора. Если директории {file}`~/bin` нет, то её сначала нужно создать:

```console
$ mkdir ~/bin
```

Затем делаем там символьную ссылку на исполняемый файл интерпретатора

```console
$ ln -s ~/julia-1.6.2/bin/julia ~/bin/julia
```

Теперь интерпретатор можно запускать, просто набрав `julia` в командной строке (возможно, нужно начать новую сессию в терминале).

Можно вместо домашней директории сделать символьную ссылку в системной директории (нужны права администратора):

```console
# ln -s ~/julia-1.6.2/bin/julia /usr/local/bin/julia
```


## MacOS

**1. Скачайте последнюю версию интерпретатора**

Скачайте .dmg-пакет с официального сайта загрузок {{ urldownload }}, {guilabel}`MacOS → 64-bit`;

**2. Установить интерпретатор**

Стандартно для .dmg-пакетов установите (перетащить .app-файл в директорию Applications).

Julia уже установлена, её можно вызвать стандартным способом, но лучше настроить работу через Terminal.app.

Откройте Terminal.app и вызовите Julia по абсолютному пути (далее команды терминала помечаются `%`, инструкция для Julia 1.6, для других версий аналогично):

```console
% /Applications/Julia-1.6.app/Contents/Resources/julia/bin/julia --version
```

Если вы получили в ответ `julia version 1.6.2`, то всё в порядке.

**3. Настройка для быстрого вызова интерпретатора**

Чтобы в будущем доступаться до Julia по короткой команде `julia`, можно добавить либо [ссылку](https://julialang.org/downloads/platform/#macos), либо создать для Julia alias в rc-файле командной оболочки.

Ниже показан способ с alias.

::::{admonition} Как найти rc-файл
:class: dropdown

Узнайте ваш `shell`

```console
% echo $SHELL
/bin/zsh
```

- Если это `zsh`, то файл для настройки {file}`~/.zshrc`.
    - Если файл отсутствует, создайте его `touch ~/.zshrc`.
- Если это `bash`, то файл для настройки {file}`~/.bashrc`.
    - Если файл отсутствует, создайте его `touch ~/.bashrc`.
::::

Добавление `alias` (для `bash` аналогично):

```console
% open ~/.zshrc
```

В открывшемся редакторе будет содержимое {file}`~/.zshrc`, добавьте в файл строчку

```console
# Alias для быстрого доступа к julia
alias julia='/Applications/Julia-1.6.app/Contents/Resources/julia/bin/julia'
```

и сохраните {file}`~/.zshrc`. Далее сделайте завершение настройки.

```console
% source ~/.zshrc
% julia --version
julia version 1.6.2
```

## Windows 10

**1. Скачайте последнюю версию интерпретатора**

На сайте загрузок {{ urldownload }} найти в таблице свою систему и архитектуру
(скорее всего, подойдет {guilabel}`Windows → 64-bit (installer)`)

**2. Установить интерпретатор**

Следуйте инструкциям установщика. По умолчанию установка идёт в пользовательскую директорию. Если хочется установить в {file}`C:\Program Files`, не забудьте запустить установщик через {guilabel}`ПКМ → Запуск от имени администратора`.

В конце установки отметить галочкой пункт **Add Julia to `PATH`**.

**3. Запуск**

Теперь интерпретатор можно запускать из командной строки командой `julia`.
*Рекомендуется* в качестве командной строки использовать современный {{ wintermdwnld }}.
