```{eval-rst}
.. meta::
   :description: Задачи и упражения на тему численного решения обыкновенных дифференциальных уравнений.
   :keywords: задача, упражнение, задание, метод верле, оду, обыкновенное дифференциальное уравнение, дифуры, вычислительная математика, вычматы
```

# Задания

## Решение системы ОДУ

Напишите реализацию явного 4-этапного метода Рунге-Кутты для решения задачи Коши для системы уравнений (см. {eq}`ode_rk_system_k`).

Проверьте метод, решив следующие задачи.

**Задача 1.**


```{math}
\begin{split}
\mathbf{u}'
&=\begin{bmatrix}
u'_1\\
u'_2
\end{bmatrix}
= \begin{bmatrix}
u_1 + u_2 \\
u_1 + u_2 + t
\end{bmatrix}
= \mathbf{f}(t, \mathbf{u}),\quad t \in (0, 1],
\\
\mathbf{u}(0) &= \begin{bmatrix} 0\\0 \end{bmatrix}.
\end{split}
```

**Задача 2.**

```{math}
\begin{split}
u'' + u' - 2t &= 0, \quad t\in(0, 1],\\
u'(0) = u(0) &= 0.
\end{split}
```

## Эллиптическая орбита

В этом задании вам предстоит рассчитать эллиптическую орбиту тела массы $m = 1$ в гравитационном поле с (неподвижным) источником в $\mathbf{r}_\text{source} = [0, 0]^\top$ и массой $m_\text{source} = 1$. Второй закон Ньютона для тела запишется в виде

```{math}
:label: ode_ex_grav_eq

\frac{\diff^2 \mathbf{r}}{\diff t^2} = -\frac{1}{|\mathbf{r}|^2}\frac{\mathbf{r}}{|\mathbf{r}|},
```

где $\mathbf{r} = [r_1, r_2]^\top$ -- радиус-вектор тела, а $|\mathbf{r}|$ -- его длина.

Отличительным свойством задачи является наличие периодических решений. Eсли 

```{math}
:label: ode_ex_grav_cond_periodic

\frac{|\mathbf{v}|^2}{2} - \frac{1}{|\mathbf{r}|} = \varepsilon < 0,
```

где $\mathbf{v}$ -- скорость тела, то решение {eq}`ode_ex_grav_eq` периодическое с периодом

```{math}
T = 2 \pi a^{3/2},\: a = - \frac{1}{2 \varepsilon}.
```

Другими словами, при соблюдении условия {eq}`ode_ex_grav_cond_periodic` орбита эллиптическая (если начальная скорость не направлена к источнику).

**Задание**

1. Подберите такое начальное условие, чтобы решением {eq}`ode_ex_grav_eq` была эллиптическая орбита. Рассчитайте один период орбиты явным 4-х этапным методом Рунге-Кутты (РК4). Исследуйте, при каких шагах интегрирования $\tau$ орбита замыкается. Приведите один график $(r_1(t), r_2(t))$ c орбитами для нескольких шагов $\tau$.

2. Реализуйте [метод Верле (*velocity Verlet*)](https://en.wikipedia.org/wiki/Verlet_integration#Velocity_Verlet). Данный метод используется для решения гамильтоновых систем, распространён в атомистическом моделировании и имеет второй порядок аппроксимации. Рассчитайте один период орбиты, исследуйте, при каких шагах интегрирования орбита замыкается или прецессирует. Также приведите один график, как в п. 1.

3. Схема Верле, в отличие от РК4 является *консервативной по энергии*. Повторите расчёты обоими методами, но уже на больших времёнах (десятки $T$). Для одинаковых начальных условий ожидается, что орбита, рассчитанная методом РК4 будет сжиматься (система теряет энергию), а орбита, рассчитанная методом Верле, будет в общем случае прецессировать. Приведите два графика, отражающие описанные эффекты: (а) орбита, рассчитанная методом Верле, (б) орбита, рассчитанная РК4.

4. Для решений из п. 3 подсчитайте зависимость полной энергии системы от времени $E(t)$. Ожидается, что в случае метода Верле энергия будет колебаться у значения в начальный момент времени $E(0)$, а в случае метода РК4 -- убывать. Приведите один график с двумя зависимостями: (а) $E(t)$ в методе Верле и (б) $E(t)$ в методе РК4.
