```{eval-rst}
.. meta::
   :description: В этом разделе мы рассмотрим методы численного решения обыкновенных дифференциальных уравнений.
   :keywords: задача коши, оду, обыкновенное дифференциальное уравнение, дифуры, вычислительная математика, вычматы
```

# Обыкновенные дифференциальные уравнения

В данном разделе рассматривается решение обыкновенных дифференциальных уравнений. В частности, мы рассмотрим **задачу Коши** для уравнения

```{index} задача Коши; Коши задача
```

```{math}
:label: ode_cauchy_scalar

\begin{split}
\frac{\diff u}{\diff t}(t) &= f(t, u(t)),\quad t \in (0, T],\\
u(0) &= u_0.
\end{split}
```

и для системы уравнений

```{math}
\frac{\diff \mathbf{u}}{\diff t}(t) &= \mathbf{f}(t, \mathbf{u}(t)), \quad t \in (0, T],\\
\mathbf{u}(0) &= \mathbf{u}_0.
```

Для краткости, мы будем также использовать запись

```{math}
\begin{split}
u'(t) &= f(t, u(t)),\\
\mathbf{u}'(t) &= \mathbf{f}(t, \mathbf{u}(t)).
\end{split}
```

Существует несколько классов методов численного решения задачи Коши. Среди них можно выделить многошаговые методы и методы Рунге-Кутты.

Будем обозначать $u(t)$ (точное) решение задачи Коши {eq}`ode_cauchy_scalar`, а $y(t)$ *приближённое* решение той же задачи.

Также введём *равномерную сетку* по переменной $t$ с шагом $\tau > 0$

```{math}
:label: ode_grid

\omega_t = \{t_i = i \tau,\: i = 0, 1, 2, \ldots \}.
```

С помощью сетки введём обозначения для точного и приближённого решения в точке $t_i$

```{math}
\begin{split}
u_i &= u(t_i),\\
y_i &= y(t_i).
\end{split}
```
