```{eval-rst}
.. meta::
   :description: Постановка задачи фазового равновесия типа пар-жидкость в изобарно-изотермической постановке.
   :keywords: pt, изобарный, фазовое равновесие, пар, жидкость, смесь, термодинамика, термодинамика смесей
```

# Постановка задачи

Сначала рассмотрим общую постановку задачи двухфазного равновесия пар-жидкость.
Пусть известны количество вещества компонентов в смеси $\mathbf{N} = [N_1, \dots, N_n]^\top$, приложенное к смеси давление $P$ и температура $T$.
Такие условия можно интерпретировать как закрытую систему (нет обмена веществом с окружающей средой), вещество которой помещено в термостат под поршень (см. {numref}`npt-problem-1-phase` и {numref}`npt-problem-2-phase`).

::::{grid}
:::{grid-item}
```{figure} static/npt-problem-1-phase.svg
---
height: 320px
name: npt-problem-1-phase
---

Однофазное состояние.
```
:::
:::{grid-item}
```{figure} static/npt-problem-2-phase.svg
---
height: 320px
name: npt-problem-2-phase
---

Сосуществующие пар и жидкость.
```
:::
::::

Чтобы решить задачу двухфазного равновесия, проверим сначала _устойчивость однофазного состояния_.
Если проверка показывает устойчивость состояния, то задача решена: в равновесии существует одна фаза, а её состав равен $\mathbf{N}$.
В противном случае мы предполагаем (в рамках нашей задачи), что в равновесии сосуществуют две фазы: пар и жидкость.
Равновесное состояние определяется _минимумом свободной энергии Гиббса_, в нашем случае энергия состояния состоит из вкладов энергий паровой и жидкой фаз
```{math}
G^{II} = G_{\vapor} + G_{\liquid}
= G(\mathbf{N}_{\vapor}, P, T) + G(\mathbf{N}_{\liquid}, P, T),
```
где энергия каждой фазы задаётся соотношением
```{math}
:label: eq:gibbs_energy_for_phase
G_{\alpha} = \sum_{i} N_{\alpha,i} \times \mu_{i}(\mathbf{N}_{\alpha}, P, T).
```
Здесь $i \in \{1, \dots, n\}$ это индекс компонента, $\alpha \in \{\vapor, \liquid\}$ это индекс фазы, а $N_{\alpha,i}$ это количество вещества компонента $i$ в фазе $\alpha$.

Поскольку мы предполагаем отсутствие химических превращений, то количество вещества каждого компонента $N_i$ сохраняется, т.е. $\mathbf{N}_{\vapor} + \mathbf{N}_{\liquid} = \mathbf{N}$.
Воспользуемся этим и получим, что энергия $G^{II}$ является функцией $n$ переменных (давление и температура постоянны)
```{math}
G^{II}(\mathbf{N}_{\vapor}, P, T)
= G(\mathbf{N}_{\vapor}, P, T) + G(\mathbf{N} - \mathbf{N}_{\vapor}, P, T).
```

Сама задача фазового равновесия сводится к задаче условной минимизации
```{math}
:label: eq:problem_as_min
\mathbf{N}^*_{\vapor} = \argmin\limits_{\mathbf{N}_{\vapor}} G^{II}(\mathbf{N}_{\vapor}, P, T), \quad P = \const, T = \const,
```
по составу паровой фазы $\mathbf{N}_{\vapor}$.
При этом составы должны удовлетворять некоторым условиям, например, положительности количеств вещества каждого компонента, балансу распределения вещества по фазам и, возможно, условиями, накладываемыми уравнением состояния (моделью флюида).

Мы не будем решать задачу равновесия в виде минимизации, а воспользуемся _необходимым условием_ минимума --- равенство нулю градиента $\partial G^{II} / \partial N_{\text{пар},i} = 0$.
Это условие приводит к равенству химпотенциалов компонентов в паре и жидкости
```{math}
\begin{split}
  \frac{\partial G^{II}}{\partial N_{\vapor,i}}
  &= \frac{\partial G_{\vapor}}{\partial N_{\vapor,i}} + \frac{\partial G_{\liquid}}{\partial N_{\vapor,i}}
  = \mu_{i}(\mathbf{N}_{\vapor}, P, T) + \frac{\partial G_{\liquid}}{\partial N_{\liquid,i}} \frac{\partial N_{\liquid,i}}{\partial N_{\vapor,i}}
  \\ &= \mu_{i}(\mathbf{N}_{\vapor}, P, T) - \mu_{i}(\mathbf{N}_{\liquid}, P, T) = 0.
\end{split}
```

Так, задача минимизации {eq}`eq:problem_as_min` свелась к решению нелинейной системы
```{math}
:label: eq:problem_as_nlsystem
  \mu_{\vapor,i} = \mu_{\liquid,i},\quad i = 1, \dots, n.
```
Однако, стоит помнить, что в общем случае решения задач {eq}`eq:problem_as_min` и {eq}`eq:problem_as_nlsystem` разные.
Например, решение задачи в виде нелинейной системы может содержать седловые точки.

## Составы и мольные доли

В системе {eq}`eq:problem_as_nlsystem` присутствуют только химпотенциалы компонентов в фазах.
Химпотенциал в переменных $\mathbf{N}$, $P$ и $T$ является однородной функцией нулевого порядка по $\mathbf{N}$, т.е.
```{math}
\mu_i(\mathbf{N}, P, T) = \mu_i(\boldsymbol{\chi}, P, T),
```
где $\boldsymbol{\chi}$ это вектор из мольных долей компонентов (в фазе)
```{math}
\chi_i
= \frac{\text{кол-во вещества компонента $i$ в фазе}}{\text{общее кол-во вещества в фазе}}
= \frac{N_i}{\sum N_i}.
```

В задаче парожидкостного равновесия устоялись следующие обозначения составов в мольных долях
```{math}
\mathbf{z} = (\text{состав в мол. долях смеси в целом}) = \frac{\mathbf{N}}{\sum_i N_i},
```
```{math}
\mathbf{x} = (\text{состав в мол. долях жидкой фазы}),
```
```{math}
\mathbf{y} = (\text{состав в мол. долях паровой (газовой) фазы}).
```
А для обозначения долей жидкой и паровой фаз используются
```{math}
  L = (\text{мол. доля жидкой фазы})
  = \frac{N_{\liquid}}{\sum_{\alpha} N_{\alpha}}
  = \frac{N_{\liquid}}{\sum_{i} N_{i}},
```
```{math}
  V = (\text{мол. доля паровой фазы})
  = \frac{N_{\vapor}}{\sum_{\alpha} N_{\alpha}}
  = \frac{N_{\vapor}}{\sum_{i} N_{i}},
```
где $N_{\alpha} = \sum_i N_{\alpha,i}$ это количество вещества, содержащегося в фазе.


## Постановка задачи в виде нелинейной системы

С учётом введённых обозначений, задача парожидкостного равновесия ставится следующим образом.

```{proof:definition} Задача расчёта двухфазного равновесия пар-жидкость
Пусть дана смесь из $n$ компонентов с мольным составом $\mathbf{z} = [z_1, \ldots, z_n]^\top$ при температуре $T$ и давлении $P$.
Необходимо определить, одна или две фазы сосуществуют в термодинамическом равновесии.
Если фазы две, то для каждой из них нужно определить долю ($L$, $V$) и состав ($\mathbf{x}$, $\mathbf{y}$).
```

Как было сказано ранее, число фаз определяется из проверки термодинамической устойчивости.
В случае двухфазного состояния имеет место система
```{math}
:label: eq:nlsystem:chempot
\mu_{\vapor,i} = \mu_{\liquid,i},\quad i = 1,\dots,n, \quad \text{($n$ уравнений)}
```
```{math}
:label: eq:nlsystem:sum_x
\sum_{i} x_i = 1, \quad \text{(1 уравнение)}
```
```{math}
:label: eq:nlsystem:sum_y
\sum_{i} y_{i} = 1, \quad \text{(1 уравнение)}
```
```{math}
:label: eq:nlsystem:LV
L + V = 1, \quad \text{(1 уравнение)}
```
```{math}
:label: eq:nlsystem:comp_balance
x_{i} L + y_{i} V = z_{i}, \quad \text{($n$ уравнений)}
```
Эта система содержит $2n + 2$ неизвестных ($x_{i}$, $y_{i}$, $V$ и $L$), а уравнений $2 n + 3$.
Однако, уравнения баланса вещества {eq}`eq:nlsystem:sum_x`-{eq}`eq:nlsystem:comp_balance` связаны между собой, поэтому можно одно отбросить, например, {eq}`eq:nlsystem:sum_x`.
