```{eval-rst}
.. meta::
   :description: Постановка задачи проверки термодинамической устойчивости фазы в изобарно-изотермической постановке.
   :keywords: устойчивость, pt, изобарный, фазовое равновесие, пар, жидкость, смесь, термодинамика, термодинамика смесей
```

# Постановка задачи

Пусть дана смесь из $n$ компонентов с мольным составом $\boldsymbol{N} = [N_1, \ldots, N_n]^\top$ при температуре $T$ и давлении $P$. Необходимо определить, является ли однофазное состояние смеси термодинамически стабильным.

## Критерий стабильности

Идея решения задачи заключается в сравнении энергий двух систем

- однофазная (исходная) система;
- двухфазная система, где вторая фаза представлена в малом количестве.

Также можно сказать, что мы попытаемся найти флуктуацию (состава и плотности), приводящую систему к двухфазному состоянию, имеющему энергию меньше, чем у однофазной системы. В этом случае однофазное состояние нестабильно. Если же такой флуктуации не существует, то однофазное состояние термодинамически стабильно.

Рассмотрим собственный потенциал NPT системы -- свободную энергию Гиббса[^gibbs_derivation]

```{math}
G = G(\boldsymbol{N}, P, T) = \sum_i N_i \mu_i(\boldsymbol{N}, P, T).
```

В дальнейшем зависимость от давления $P$ и температуры $T$ писать не будем -- это параметры задачи.

[^gibbs_derivation]: Выражение свободной энергии Гиббса для смеси следует из аддитивности внутренней энергии и энтропии. Подробнее об этом в {cite}`Brusilovsky2002` (Раздел 2.9, Химический потенциал).

Теперь предположим, что часть вещества $\boldsymbol{\varepsilon}$ исходной системы образует вторую фазу. Тогда разница энергий двухфазного и однофазного состояний будет
```{math}
:label: ptstab_gibbs_delta

\Delta G = G(\boldsymbol{N} - \boldsymbol{\varepsilon}) + G(\boldsymbol{\varepsilon}) - G(\boldsymbol{N})
```

Если получится найти такое количество вещества $\boldsymbol{\varepsilon}$, что $\Delta G < 0$, то двухфазное состояние имеет меньшую энергию, а однофазное состояние -- нестабильно. Для этого, можно, например, найти глобальный минимум $\Delta G$ и посмотреть его знак. Однако, распространён следующий метод.

Будем считать, что общее количество вещества во второй фазе $\|\boldsymbol{\varepsilon}\|$ мало (вторая фаза в зародышевом состоянии, например). Разложим тогда в ряд Тейлора первое слагаемое из уравнения {eq}`ptstab_gibbs_delta`
```{math}
G(\boldsymbol{N} - \boldsymbol{\varepsilon}) \approx G(\boldsymbol{N}) - \sum_i \varepsilon_i \frac{\partial G}{\partial N_i} (\boldsymbol{N})
= G(\boldsymbol{N}) - \sum_i \varepsilon_i \mu_i(\boldsymbol{N}).
```

С учётом разложения, получаем[^gibbs_delta_approx]
```{math}
\Delta G = \sum_i \varepsilon_i \mu_i(\boldsymbol{\varepsilon}) - \sum_i \varepsilon_i \mu_i(\boldsymbol{N})
= \sum_i \varepsilon_i (\mu_i(\boldsymbol{\varepsilon}) - \mu_i(\boldsymbol{N})).
```

[^gibbs_delta_approx]: Строго говоря, это равенство выполняется приближённо. Знак точного равенства поставлен с целью упрощения изложения и математической записи.

Избавимся от абсолютных величин, введя общее количество вещества в фазе-зародыше $\varepsilon = \sum_i \varepsilon_i$
```{math}
:label: ptstab_gibbs_delta_relative

\Delta G = \varepsilon \sum_i x_i (\mu_i(\boldsymbol{x}) - \mu^0_i),
```
здесь $\boldsymbol{x} \equiv \boldsymbol{\varepsilon} / \varepsilon$ -- мольный состав фазы-зародыша, а $\mu^0_i$ -- химпотенциал компонентна в однофазном состоянии.

Поскольку в {eq}`ptstab_gibbs_delta_relative` $\varepsilon > 0$, получаем критерий стабильности фазы
```{math}
:label: ptstab_tpd

F(\boldsymbol{x}) = \sum_i x_i (\mu_i(\boldsymbol{x}) - \mu^0_i) \ge 0.
```



## Математическая постановка

Итак, в общем случае, необходимо перебрать всевозможные составы $\boldsymbol{x}$ и проверить в каждом из них знак функции $F(\boldsymbol{x})$ {eq}`ptstab_tpd`. Конечно, можно поступить и проще, найдя глобальный минимум $F$, и посмотреть знак $F$ в нём. Вместо поиска минимума можно проверить знак $F$ в стационарных точках {cite}`baker_1982`.

Условие стационарности $\nabla F = \boldsymbol{0}$ с учётом $\sum_i x_i = 1$ приводит к системе уравнений
```{math}
:label: ptstab_chempot_stationary_system

\mu_i(\boldsymbol{x}) -\mu^0_i = \lambda, \quad i=1,\dots,n
```
где $\lambda$ -- константа. Так, в стационарной точке $F = \lambda$.

```{index} летучесть
```
```{index} коэффициент ; летучести
```
```{index} see: фугитивность ; летучесть
```
Как правило, работают не с самим химпотенциалом, а с **летучестью** (фугитивностью) $f_i$ или **коэффициентом летучести** $\varphi_i \equiv f_i / (x_i P)$.

По определению, летучесть $f_i$ связана с химпотенциалом $\mu_i$ так[^fugacity_introduced]
```{math}
:label: fugacity_definition

\mu_i = \hat{\mu}(T) + R T \ln \bigg( \frac{f_i}{[\text{ед. давления}]} \bigg),
```
здесь $\hat{\mu}(T)$ -- химический потенциал идеального газа при единичном (референсном) давлении.
Фактически, это поправка, зависящая только от температуры и не влияющая на утверждения о разности химических потенциалов в изотермических состояниях.
Поскольку летучесть $f_i$ имеет размерность давления, под знаком логарифма в знаменателе стоит единица давления для обезразмеривания.
Из определения следует, что для смеси идеальных газов $V \to \infty$, а $f_i = x_i P$ и $\varphi_i = 1$.

[^fugacity_introduced]: Термодинамическая величина "летучесть" (фугитивность, англ. *fugactity*) введена Г. Льюисом в 1901 году для сохранения простоты связи между изотермическим изменением химпотенциала и способности вещества переходить из одной системы в другую (физико-химические равновесия).

В свою очередь, для коэффициента летучести $\varphi_i \equiv f_i / (x_i P)$ {eq}`fugacity_definition` перепишется в виде
```{math}
\mu_i = \hat{\mu}(T) + R T (\ln{x_i} + \ln{\varphi_i} + \ln{P}).
```

Если вернёмся теперь к разности химпотенциалов {eq}`ptstab_chempot_stationary_system`, то получим систему на мольный состав тестовой фазы $\boldsymbol{x}$
```{math}
\begin{aligned}
\ln{x_i} + \ln{\varphi_i(\boldsymbol{x})} - (\ln{z_i} + \ln{\varphi_i(\boldsymbol{z})})
&= \frac{\lambda}{R T}
\\ \ln{x_i} + \ln{\varphi_i(\boldsymbol{x})} - h_i &= k
\\ i &= 1,\dots,n
\end{aligned}
```
здесь введены обозначения $z_i$ -- мольная доля компонента $i$ в исходной смеси, $h_i \equiv \ln{z_i} + \ln{\varphi_i(\boldsymbol{z})}$ и $k \equiv \lambda/ R T$.

Последним шагом сведём систему к однородной. Для этого введём замену переменных $x_i = X_i e^k$, тогда
```{math}
:label: ptstab_chempot_stationary_system_final

\ln{X_i} + \ln{\varphi_i(\boldsymbol{x}(\boldsymbol{X}))} - h_i = 0,\quad i = 1,\dots,n
```
Формально, новые переменные $X_i$ можно считать составами, поскольку
```{math}
:label: ptstab_new_variables_molfrac_property

x_i = \frac{X_i}{\sum_i X_i}.
```

Итак, проверку термодинамической стабильности однофазного состояния мы свели к

- нахождению решений системы уравнений {eq}`ptstab_chempot_stationary_system_final` и
- последующей проверкой критерия {eq}`ptstab_tpd` в каждом решении.

Если при этом хотя бы для одного решения оказывается $F < 0$, то однофазное состояние смеси нестабильно.
